package java.awt;

import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleAction;
import javax.accessibility.AccessibleContext;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.awt.event.ItemEvent;
import java.util.EventListener;
import java.awt.peer.ChoicePeer;
import java.awt.event.ItemListener;
import java.util.Vector;
import javax.accessibility.Accessible;

public class Choice extends Component implements ItemSelectable, Accessible
{
    Vector<String> pItems;
    int selectedIndex;
    transient ItemListener itemListener;
    private static final String base = "choice";
    private static int nameCounter;
    private static final long serialVersionUID = -4075310674757313071L;
    private int choiceSerializedDataVersion;
    
    public Choice() throws HeadlessException {
        this.selectedIndex = -1;
        this.choiceSerializedDataVersion = 1;
        GraphicsEnvironment.checkHeadless();
        this.pItems = new Vector();
    }
    
    @Override
    String constructComponentName() {
        synchronized (Choice.class) {
            return "choice" + Choice.nameCounter++;
        }
    }
    
    @Override
    public void addNotify() {
        synchronized (this.getTreeLock()) {
            if (this.peer == null) {
                this.peer = this.getToolkit().createChoice(this);
            }
            super.addNotify();
        }
    }
    
    public int getItemCount() {
        return this.countItems();
    }
    
    @Deprecated
    public int countItems() {
        return this.pItems.size();
    }
    
    public String getItem(int n) {
        return this.getItemImpl(n);
    }
    
    final String getItemImpl(int index) {
        return (String)this.pItems.elementAt(index);
    }
    
    public void add(String s) {
        this.addItem(s);
    }
    
    public void addItem(String s) {
        synchronized (this) {
            this.insertNoInvalidate(s, this.pItems.size());
        }
        this.invalidateIfValid();
    }
    
    private void insertNoInvalidate(String obj, int index) {
        if (obj == null) {
            throw new NullPointerException("cannot add null item to Choice");
        }
        this.pItems.insertElementAt(obj, index);
        ChoicePeer choicePeer = (ChoicePeer)this.peer;
        if (choicePeer != null) {
            choicePeer.add(obj, index);
        }
        if (this.selectedIndex < 0 || this.selectedIndex >= index) {
            this.select(0);
        }
        return;
    }
    
    public void insert(String s, int min) {
        synchronized (this) {
            if (min < 0) {
                throw new IllegalArgumentException("index less than zero.");
            }
            min = Math.min(min, this.pItems.size());
            this.insertNoInvalidate(s, min);
        }
        this.invalidateIfValid();
    }
    
    public void remove(String s) {
        synchronized (this) {
            int index = this.pItems.indexOf(s);
            if (index < 0) {
                throw new IllegalArgumentException("item " + s + " not found in choice");
            }
            this.removeNoInvalidate(index);
        }
        this.invalidateIfValid();
    }
    
    public void remove(int n) {
        synchronized (this) {
            this.removeNoInvalidate(n);
        }
        this.invalidateIfValid();
    }
    
    private void removeNoInvalidate(int index) {
        this.pItems.removeElementAt(index);
        ChoicePeer choicePeer = (ChoicePeer)this.peer;
        if (choicePeer != null) {
            choicePeer.remove(index);
        }
        if (this.pItems.size() == 0) {
            this.selectedIndex = -1;
        }
        else if (this.selectedIndex == index) {
            this.select(0);
        }
        else if (this.selectedIndex > index) {
            this.select(this.selectedIndex - 1);
        }
    }
    
    public void removeAll() {
        synchronized (this) {
            if (this.peer != null) {
                ((ChoicePeer)this.peer).removeAll();
            }
            this.pItems.removeAllElements();
            this.selectedIndex = -1;
        }
        this.invalidateIfValid();
    }
    
    public synchronized String getSelectedItem() {
        return (this.selectedIndex >= 0) ? this.getItem(this.selectedIndex) : null;
    }
    
    @Override
    public synchronized Object[] getSelectedObjects() {
        if (this.selectedIndex >= 0) {
            return new Object[] { this.getItem(this.selectedIndex) };
        }
        return null;
    }
    
    public int getSelectedIndex() {
        return this.selectedIndex;
    }
    
    public synchronized void select(int n) {
        if (n >= this.pItems.size() || n < 0) {
            throw new IllegalArgumentException("illegal Choice item position: " + n);
        }
        if (this.pItems.size() > 0) {
            this.selectedIndex = n;
            ChoicePeer choicePeer = (ChoicePeer)this.peer;
            if (choicePeer != null) {
                choicePeer.select(n);
            }
        }
        return;
    }
    
    public synchronized void select(String o) {
        int index = this.pItems.indexOf(o);
        if (index >= 0) {
            this.select(index);
        }
    }
    
    @Override
    public synchronized void addItemListener(ItemListener itemListener) {
        if (itemListener == null) {
            return;
        }
        this.itemListener = AWTEventMulticaster.add(this.itemListener, itemListener);
        this.newEventsOnly = true;
        return;
    }
    
    @Override
    public synchronized void removeItemListener(ItemListener itemListener) {
        if (itemListener == null) {
            return;
        }
        this.itemListener = AWTEventMulticaster.remove(this.itemListener, itemListener);
        return;
    }
    
    public synchronized ItemListener[] getItemListeners() {
        return (ItemListener[])this.getListeners(ItemListener.class);
    }
    
    @Override
    public <T extends EventListener> T[] getListeners(Class<T> clazz) {
        if (clazz == ItemListener.class) {
            return AWTEventMulticaster.getListeners(this.itemListener, clazz);
        }
        return super.getListeners(clazz);
    }
    
    @Override
    boolean eventEnabled(AWTEvent awtEvent) {
        if (awtEvent.id == 701) {
            return (this.eventMask & 0x200L) != 0x0L || this.itemListener != null;
        }
        return super.eventEnabled(awtEvent);
    }
    
    @Override
    protected void processEvent(AWTEvent awtEvent) {
        if ((awtEvent instanceof ItemEvent)) {
            this.processItemEvent((ItemEvent)awtEvent);
            return;
        }
        super.processEvent(awtEvent);
        return;
    }
    
    protected void processItemEvent(ItemEvent itemEvent) {
        ItemListener itemListener = this.itemListener;
        if (itemListener != null) {
            itemListener.itemStateChanged(itemEvent);
        }
    }
    
    @Override
    protected String paramString() {
        return super.paramString() + ",current=" + this.getSelectedItem();
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        AWTEventMulticaster.save(objectOutputStream, "itemL", this.itemListener);
        objectOutputStream.writeObject(null);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException, HeadlessException {
        GraphicsEnvironment.checkHeadless();
        objectInputStream.defaultReadObject();
        Object object;
        while (null != (object = objectInputStream.readObject())) {
            if ("itemL" == ((String)object).intern()) {
                this.addItemListener((ItemListener)(ItemListener)objectInputStream.readObject());
            }
            else {
                objectInputStream.readObject();
            }
        }
    }
    
    private static native void initIDs();
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new Choice.AccessibleAWTChoice();
        }
        return this.accessibleContext;
    }
    
    static {
        Choice.nameCounter = 0;
        Toolkit.loadLibraries();
        if (!GraphicsEnvironment.isHeadless()) {
            initIDs();
        }
    }
    
    protected class AccessibleAWTChoice extends Component.AccessibleAWTComponent implements AccessibleAction
    {
        private static final long serialVersionUID = 7175603582428509322L;
        
        public AccessibleAWTChoice() {
        }
        
        @Override
        public AccessibleAction getAccessibleAction() {
            return this;
        }
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.COMBO_BOX;
        }
        
        @Override
        public int getAccessibleActionCount() {
            return 0;
        }
        
        @Override
        public String getAccessibleActionDescription(int n) {
            return null;
        }
        
        @Override
        public boolean doAccessibleAction(int n) {
            return false;
        }
    }
}
